unit uThreads;

interface
uses
  classes, imageEn, windows, SyncObjs;

type
  TWorkerThread = class(TThread)
  private
    FWorkImage: TImageEn;
    FUseLayers: Boolean;
    FWorkStartEvent, FWorkCompleteEvent: TEvent;
  public
    constructor Create(UseLayers: Boolean);
    destructor Destroy; override;

    procedure Execute; override;

    property WorkStartEvent: TEvent read FWorkStartEvent write FWorkStartEvent;
    property WorkCompleteEvent: TEvent read FWorkCompleteEvent write FWorkCompleteEvent;
  end;


  TManagerThread = class(TThread)
  private
    FTaskCount, FThreadCount: Integer;

    FThreads                : array of TThread;
    FThreadHandles          : array of THandle;
    FThreadCompletionEvents : array of THandle;
    procedure ProcessThreadResult(AThread: TWorkerThread);
  public
    constructor Create(ThreadCount, TaskCount: Integer; UseLayers: Boolean);
    destructor Destroy; override;

    procedure Execute; override;
  end;


implementation
uses
  iexBitmaps, iexLayers, sysutils, hyiedefs, hyieutils, graphics,
  ActiveX;

{ TWorkerThread }

constructor TWorkerThread.Create(UseLayers: Boolean);
begin
  inherited Create(False);

  FWorkStartEvent := TEvent.Create(nil, False, False, '');
  FWorkCompleteEvent := TEvent.Create(nil, True, True, '');

  FUseLayers := UseLayers;
  FWorkImage := TImageEn.Create(nil);
  FWorkImage.LegacyBitmap := False;
end;

destructor TWorkerThread.Destroy;
begin
  FWorkImage.Free;
  FWorkStartEvent.Free;
  FWorkCompleteEvent.Free;

  inherited;
end;

procedure TWorkerThread.Execute;
var
  AText, BText, FileName: string;
  ImageHeight, ImageWidth, LineHeight, f: Integer;
  WorkList: TStringList;
  AGUID: TGUID;
  ARect: TRect;
begin
  while not Terminated do
  begin
    WaitForSingleObject(FWorkStartEvent.Handle, INFINITE);
    if Terminated then
      Break;

    FWorkImage.IO.LoadFromFileDICOM('IM_0001.dcm');
    FWorkImage.IEBitmap.PixelFormat := ie24RGB;
    FWorkImage.IO.Params.BitsPerSample := 24;
    FWorkImage.IO.Params.SamplesPerPixel := 8;

    ImageHeight := FWorkImage.IEBitmap.Height;
    ImageWidth := FWorkImage.IEBitmap.Width;

    if FUseLayers then
    begin
      FWorkImage.LayersAdd(ielkText, 0, 0);  // top left
      TIETextLayer(FWorkImage.Layers[1]).Alignment := iejLeft;
      TIETextLayer(FWorkImage.Layers[1]).Layout := ielTop;

      TIETextLayer(FWorkImage.Layers[1]).Text := DateTimeToStr(Now);
      FWorkImage.LayersMergeAll;
      FWorkImage.Update;
    end
    else begin
      AText := DateTimeToStr(Now);
      AText := AText + #13#10 + AText + #13#10 + AText;

      FWorkImage.IEBitmap.Canvas.Font.Name := 'Arial';
      FWorkImage.IEBitmap.Canvas.Font.Size := 14;
      FWorkImage.IEBitmap.Canvas.Font.Color := clWhite;

      //  top left
      FWorkImage.Proc.TextOut(10, 10, AText, nil, 0, True, False);

      //  bottom left
      DrawText(FWorkImage.IEBitmap.Canvas.Handle, PChar(AText),
        Length(AText), ARect, DT_LEFT or DT_CALCRECT);

      FWorkImage.Proc.TextOut(10, ImageHeight - 10 - ARect.Height,
        AText, nil, 0, True, False);

      //  top right
      WorkList := TStringList.Create;
      try
        BText := AText;
        WorkList.Text := BText;
        if WorkList.Count > 0 then
        begin
          if WorkList[WorkList.Count - 1] = '' then
            WorkList.Delete(WorkList.Count - 1);

          ARect.Height := 1;
          ARect.Width := 1;

          if WorkList.Count > 0 then
          begin
            BText := WorkList.Text;
            BText := Copy(BText, 1, Length(BText) - 2);
            DrawText(FWorkImage.IEBitmap.Canvas.Handle, PChar(BText),
              Length(BText), ARect, DT_LEFT or DT_CALCRECT);
            LineHeight := ARect.Height div WorkList.Count;

            for f := 0 to WorkList.Count - 1 do
            begin
              BText := WorkList[f];
              DrawText(FWorkImage.IEBitmap.Canvas.Handle, PChar(BText),
                Length(BText), ARect, DT_RIGHT or DT_CALCRECT);

              FWorkImage.Proc.TextOut(ImageWidth - 10 - ARect.Width, 10 + (f * LineHeight),
                WorkList[f], nil, 0, True, False);
            end;
          end;
        end;
      finally
        WorkList.Free;
      end;

      CoCreateGuid(AGuid);
      FileName := GUIDToString(AGuid) + '.png';

      FWorkImage.IO.SaveToFilePNG(FileName);

      FWorkCompleteEvent.SetEvent;
    end;
  end;
end;

{ TManagerThread }

constructor TManagerThread.Create(ThreadCount, TaskCount: Integer; UseLayers: Boolean);
var
  AThread: TWorkerThread;
  f: Integer;
begin
  inherited Create(True);

  FTaskCount := TaskCount;
  FThreadCount := ThreadCount;

  SetLength(FThreads, ThreadCount);
  SetLength(FThreadHandles, ThreadCount);
  SetLength(FThreadCompletionEvents, ThreadCount);

  for f := 0 to ThreadCount - 1 do
  begin
    AThread := TWorkerThread.Create(UseLayers);
    FThreads[f] := AThread;
    FThreadHandles[f] := AThread.Handle;
    FThreadCompletionEvents[f] := AThread.WorkCompleteEvent.Handle;
  end;
end;

destructor TManagerThread.Destroy;
begin

  inherited;
end;


procedure TManagerThread.ProcessThreadResult(AThread: TWorkerThread);
begin

end;

procedure TManagerThread.Execute;
var
  TaskIndex: Integer;
  WaitResult: Cardinal;
  AThread: TWorkerThread;
begin
  TaskIndex := 0;
  while TaskIndex < FTaskCount do
  begin
    WaitResult := WaitForMultipleObjects(FThreadCount, @FThreadCompletionEvents[0], False, INFINITE);
    if WaitResult < (WAIT_OBJECT_0 + FThreadCount) then
    begin
      AThread := TWorkerThread(FThreads[WaitResult]);
      ProcessThreadResult(AThread);

      Inc(TaskIndex);
      if TaskIndex < FTaskCount then
      begin
        AThread.WorkCompleteEvent.ResetEvent;
        AThread.WorkStartEvent.SetEvent;
      end
      else begin
        AThread.Terminate;
        AThread.WorkStartEvent.SetEvent;
      end;
    end;
  end;

  WaitResult := WaitForMultipleObjects(FThreadCount, @FThreadCompletionEvents[0], True, INFINITE);
end;

end.
